@@ -2,7 +2,7 @@
 """
 import math
 
-from .db import CorpsEval
+from .db import CorpsEval, C104, C103
 from db_hj3415 import mongo2
 from util_hj3415 import utils
 from krx_hj3415 import krx
@@ -18,19 +18,16 @@ formatter = logging.Formatter('%(levelname)s: [%(name)s] %(message)s')
 ch = logging.StreamHandler()
 ch.setFormatter(formatter)
 logger.addHandler(ch)
-logger.setLevel(logging.WARNING)
+logger.setLevel(logging.INFO)
 
 # 주식을 통한 기대수익률 - 금리가 3%일 경우 두배인 6% 정도로 잡는다.
 EXPECT_EARN = 0.04
 
 
 def red(client, code: str) -> dict:
-    if utils.is_6digit(code):
-        corps_eval = CorpsEval(client, code)
-        c101 = mongo2.C101(client, code)
-        c103 = mongo2.C103(client, code, 'c103재무상태표q')
-    else:
-        raise ValueError(f'Invalid code: {code}')
+    corps_eval = CorpsEval(client, code)
+    c101 = mongo2.C101(client, code)
+    c103 = C103(client, code, 'c103재무상태표q')
 
     d1, 지배주주당기순이익 = corps_eval.calc당기순이익()
     d2, 유동자산 = corps_eval.calc유동자산()
@@ -38,14 +35,16 @@ def red(client, code: str) -> dict:
     d4, 부채평가 = corps_eval.calc비유동부채()
 
     c103.page = 'c103재무상태표q'
-    d5, 투자자산 = c103.latest_value('투자자산')
-    d6, 투자부동산 = c103.latest_value('투자부동산')
+    d5, 투자자산 = c103.latest_value('투자자산', nan_to_zero=True)
+    d6, 투자부동산 = c103.latest_value('투자부동산', nan_to_zero=True)
 
     # 사업가치 계산 - 지배주주지분 당기순이익 / 기대수익률
     사업가치 = round(지배주주당기순이익 / EXPECT_EARN, 2)
 
     # 재산가치 계산 - 유동자산 - (유동부채*1.2) + 고정자산중 투자자산
     재산가치 = round(유동자산 - (유동부채 * 1.2) + 투자자산 + 투자부동산, 2)
+    if math.isnan(재산가치):
+        logger.warning(f'유동자산: {유동자산} - 유동부채: {유동부채} * 1.2 + 투자자산: {투자자산} + 투자부동산: {투자부동산}')
 
     발행주식수 = c103.latest_value('발행주식수')[1]
     if math.isnan(발행주식수):
@@ -58,7 +57,7 @@ def red(client, code: str) -> dict:
     try:
         red_price = round(((사업가치 + 재산가치 - 부채평가) * 100000000) / 발행주식수)
     except (ZeroDivisionError, ValueError) as e:
-        logger.error(f'{e} : {code}')
+        logger.error(f'In calc red price... {e} : {code}')
         red_price = float('nan')
 
     logger.debug(f'Red Price : {red_price}원')
@@ -73,15 +72,12 @@ def red(client, code: str) -> dict:
 
 
 def mil(client, code: str) -> dict:
-    if utils.is_6digit(code):
-        corps_eval = CorpsEval(client, code)
-        c103 = mongo2.C103(client, code, 'c103현금흐름표q')
-        c104 = mongo2.C104(client, code, 'c104q')
-    else:
-        raise ValueError(f'Invalid code: {code}')
+    corps_eval = CorpsEval(client, code)
+    c103 = C103(client, code, 'c103현금흐름표q')
+    c104 = C104(client, code, 'c104q')
 
     marketcap = corps_eval.marketcap
-    logger.info(f'{code} market cap: {marketcap}')
+    logger.debug(f'{code} market cap: {marketcap}')
     fcf_dict = corps_eval.findFCF()
     pfcf_dict = corps_eval.findPFCF()
     d1, 지배주주당기순이익 = corps_eval.calc당기순이익()
@@ -96,10 +92,14 @@ def mil(client, code: str) -> dict:
     주주수익률 = float('nan') if marketcap == 0 else round((재무활동현금흐름 / marketcap * -100), 2)
     이익지표 = float('nan') if marketcap == 0 else round((지배주주당기순이익 - 영업활동현금흐름) / marketcap, 5)
 
-    logger.info(f'{code} fcf_dict : {fcf_dict}')
-    logger.info(f"{code} market_cap : {marketcap}")
-    logger.info(f'{code} pfcf_dict : {pfcf_dict}')
-    logger.info(f'{code} pcr_dict : {pcr_dict}')
+    if math.isnan(주주수익률) or math.isnan(이익지표):
+        logger.warning(f'주주수익률: {주주수익률} 이익지표: {이익지표}')
+        logger.warning(f'재무활동현금흐름: {재무활동현금흐름}지배주주당기순이익: {지배주주당기순이익}영업활동현금흐름: {영업활동현금흐름}')
+
+    logger.debug(f'{code} fcf_dict : {fcf_dict}')
+    logger.debug(f"{code} market_cap : {marketcap}")
+    logger.debug(f'{code} pfcf_dict : {pfcf_dict}')
+    logger.debug(f'{code} pcr_dict : {pcr_dict}')
 
     return {
         '주주수익률': 주주수익률,
@@ -131,24 +131,16 @@ def blue(client, code: str) -> dict:
     <영업이익률>
     영업이익률은 기업의 경쟁력척도로 경쟁사에 비해 높으면 경제적해자를 갖춘셈
     """
+    corps_eval = CorpsEval(client, code)
+    c104 = C104(client, code, 'c104y')
 
-    if utils.is_6digit(code):
-        corps_eval = CorpsEval(client, code)
-        c104 = mongo2.C104(client, code, 'c104y')
-    else:
-        raise ValueError(f'Invalid code: {code}')
+    d1, 유동비율 = corps_eval.calc유동비율(pop_count=3)
+    logger.debug(f'유동비율 {유동비율} / [{d1}]')
 
-    ###############################################################
-
-    d1, 유동비율 = corps_eval.calc유동비율(nan_to_zero=False)
-    logger.info(f'유동비율 {유동비율}/{d1}')
-
-    ###############################################################
-
-    dict이자보상배율 = c104.find('이자보상배율')
-    dict순운전자본회전율 = c104.find('순운전자본회전율')
-    dict재고자산회전율 = c104.find('재고자산회전율')
-    dict순부채비율 = c104.find('순부채비율')
+    dict이자보상배율 = c104.find('이자보상배율', allow_empty=True)
+    dict순운전자본회전율 = c104.find('순운전자본회전율', allow_empty=True)
+    dict재고자산회전율 = c104.find('재고자산회전율', allow_empty=True)
+    dict순부채비율 = c104.find('순부채비율', allow_empty=True)
 
     c104.page = 'c104q'
     d2, 이자보상배율 = c104.latest_value('이자보상배율')
@@ -156,10 +148,11 @@ def blue(client, code: str) -> dict:
     d4, 재고자산회전율 = c104.latest_value('재고자산회전율')
     d5, 순부채비율 = c104.latest_value('순부채비율')
 
-    logger.info(f'이자보상배율 : {이자보상배율} {dict이자보상배율}')
-    logger.info(f'순운전자본회전율 : {순운전자본회전율} {dict순운전자본회전율}')
-    logger.info(f'재고자산회전율 : {재고자산회전율} {dict재고자산회전율}')
-    logger.info(f'순부채비율 : {순부채비율} {dict순부채비율}')
+    if len(dict이자보상배율) == 0 or len(dict순운전자본회전율) == 0 or len(dict재고자산회전율) == 0 or len(dict순부채비율) == 0:
+        logger.warning(f'이자보상배율 : {이자보상배율} {dict이자보상배율}')
+        logger.warning(f'순운전자본회전율 : {순운전자본회전율} {dict순운전자본회전율}')
+        logger.warning(f'재고자산회전율 : {재고자산회전율} {dict재고자산회전율}')
+        logger.warning(f'순부채비율 : {순부채비율} {dict순부채비율}')
 
     ################################################################
 
@@ -180,18 +173,15 @@ def growth(client, code: str) -> dict:
     <영업이익률>
     영업이익률은 기업의 경쟁력척도로 경쟁사에 비해 높으면 경제적해자를 갖춘셈
     """
-    if utils.is_6digit(code):
-        c104 = mongo2.C104(client, code, 'c104y')
-        c106y = mongo2.C106Y(client, code)
-    else:
-        raise ValueError(f'Invalid code: {code}')
+    c104 = C104(client, code, 'c104y')
+    c106y = mongo2.C106Y(client, code)
 
     dict매출액증가율 = c104.find('매출액증가율')
 
     c104.page = 'c104q'
     d1, 매출액증가율 = c104.latest_value('매출액증가율')
 
-    logger.info(f'매출액증가율 : {매출액증가율} {dict매출액증가율}')
+    logger.debug(f'매출액증가율 : {매출액증가율} {dict매출액증가율}')
 
     ################################################################
 
@@ -200,7 +190,7 @@ def growth(client, code: str) -> dict:
         dict영업이익률 = c106y.find('영업이익률')
     except KeyError:
         dict영업이익률 = {'Unnamed': float('nan')}
-    logger.info(f'{code} 영업이익률 {dict영업이익률}')
+    logger.debug(f'{code} 영업이익률 {dict영업이익률}')
 
     return {
         '매출액증가율': (매출액증가율, dict매출액증가율),
@@ -243,7 +233,7 @@ def growth(client, code: str) -> dict:
 """
 
 
-def _make_df_part(client, codes: list, q):
+def _make_df_part(db_addr, codes: list, q):
     def make_record(c: str) -> dict:
         # 장고에서 사용할 eval 테이블을 만들기 위해 각각의 레코드를 구성하는 함수
         c101 = mongo2.C101(client, c).get_recent()
@@ -273,7 +263,8 @@ def _make_df_part(client, codes: list, q):
             '매출액증가율': utils.to_float(growth_dict['매출액증가율'][0]),
             'date': list(set(mil_date + red_date + growth_date))
         }
-
+    # 각 코어별로 디비 클라이언트를 만들어야만 한다. 안그러면 에러발생
+    client = mongo2.connect_mongo(db_addr)
     t = len(codes)
     d = []
     for i, code in enumerate(codes):
@@ -320,14 +311,14 @@ def make_today_eval_df(db_addr: str, refresh: bool = False) -> pd.DataFrame:
 
         print('*' * 25, f"Eval all using multiprocess", '*' * 25)
         print(f'Total {len(codes)} items..')
-        logger.info(codes)
+        logger.debug(codes)
         n, divided_list = _code_divider(codes)
 
         start_time = time.time()
         q = Queue()
         ths = []
         for i in range(n):
-            ths.append(Process(target=_make_df_part, args=(client, divided_list[i], q)))
+            ths.append(Process(target=_make_df_part, args=(db_addr, divided_list[i], q)))
         for i in range(n):
             ths[i].start()
         df_list = []
@@ -355,13 +346,13 @@ def yield_valid_spac(client) -> tuple:
         tuple: (code, name, price)
     """
     codes = mongo2.Corps(client).get_all_corps()
-    logger.info(f'len(codes) : {len(codes)}')
+    logger.debug(f'len(codes) : {len(codes)}')
     print('<<< Finding valuable SPAC >>>')
     for i, code in enumerate(codes):
         name = krx.get_name(code)
-        logger.info(f'code : {code} name : {name}')
+        logger.debug(f'code : {code} name : {name}')
         if '스팩' in str(name):
-            logger.info(f'>>> spac - code : {code} name : {name}')
+            logger.debug(f'>>> spac - code : {code} name : {name}')
             price, _, _ = utils.get_price_now(code=code)
             if price <= 2000:
                 logger.warning(f'현재가:{price}')
